/*
 * ################################################################
 *
 * ProActive Parallel Suite(TM): The Java(TM) library for
 *    Parallel, Distributed, Multi-Core Computing for
 *    Enterprise Grids & Clouds
 *
 * Copyright (C) 1997-2011 INRIA/University of
 *                 Nice-Sophia Antipolis/ActiveEon
 * Contact: proactive@ow2.org or contact@activeeon.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License
 * as published by the Free Software Foundation; version 3 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * If needed, contact us to obtain a release under GPL Version 2 or 3
 * or a different license than the AGPL.
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package org.ow2.proactive.scheduler.examples;

import java.util.Random;


/**
 * NativeTestWithRandomDefault is a class that should test native process generating random error code.
 *
 * @author The ProActive Team
 * @since ProActive Scheduling 0.9.1
 */
public class NativeTestWithRandomDefault {

    /**
     * Task that will wait randomly from 5 to 1 minute
     * And will then return an error code 3 times on 5.
     *
     * @param args
     */
    public static void main(String[] args) {
        int maxSleepTime = 30;
        if (args.length > 0) {
            if (args[0].equals("final")) {
                System.out.println("Final Task started !");
            } else {
                maxSleepTime = 1;
            }
        }
        Random time = new Random(System.currentTimeMillis());
        int sleepTime = time.nextInt(maxSleepTime) + 1;
        try {
            Thread.sleep(sleepTime * 1000);
        } catch (InterruptedException e) {
        }
        System.out.println("Slept for " + sleepTime + " seconds");
        Random exit = new Random(System.currentTimeMillis());
        int exitStatus = exit.nextInt(5);
        if (exitStatus >= 3) {
            System.out.println("Exit code is : " + (exitStatus - 1));
            System.exit(exitStatus - 1);
        } else {
            System.out.println("Exit code is : 0");
            System.exit(0);
        }
    }

}
